
<#
.SYNOPSIS
    A PowerShell wrapper for extracting files using 7Zip's 7z.exe command line.

.DESCRIPTION
    1. Detects the 7Zip executable. If it's not in the default location of "C:\Program Files\7-zip\7z.exe", it will ask the user for the location.
    2. Gets the contents of $FilesPath
    3. Executes 7z.exe -x -y <Filename> on each file in $FilesPath
    
.PARAMETER FilesPath
    The folder containing the driver archives you wish to decompress / de-archive

.PARAMETER DestinationPath
    The folder where you'd like to unpack the driver files to.

.EXAMPLE
    By itself:
    Expand-DriverArchive -FilesPath 'E:\WDK\Drivers\Installers\Dell\T5600' -DestinationPath 'E:\WDK\Drivers\Dell\T5600'

.NOTES
    Author:    Cameron Ratchford
    Date:      January 9, 2020  
#>
function Expand-DriverArchive
{
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)]
        [string]$FilesPath,
        [Parameter(Mandatory=$true)]
        [string]$DestinationPath
    )

    ###### Vars ######
    $7zipPath = "C:\Program Files\7-zip\"
    $7zipFullPath = $7zipPath + "7z.exe"

    ###### Tests ######
    # Test to see if 7zip is installed
    if (Test-Path -Path $7zipPath)
    {
        Write-Host -ForegroundColor Green "7zip install decetected"
    }
    else 
    {
        Write-Warning -Message "7z.exe not found in $7zipFullPath`n"
        $7zipPath = Read-Host -Prompt "Please specify the path that contains 7z.exe file:`n"

        if (Test-Path -Path $7zipPath)
        {
            Write-Host -ForegroundColor Green "7zip install decetected"
        }
        else 
        {
            Write-Error -Message "7z.exe still not found`nPlease install 7zip and run the script again."
            exit(1)
        }
    }

    # Test to see if the FilesPath contains compatible archives
    $Files = Get-ChildItem -Path $FilesPath
    
    foreach ($File in $Files)
    {
        $FileName = $File.FullName
        $NewDir = $File.Name

        $7zip_params = @{
            ScriptBlock = { C:\Program` Files\7-zip\7zip.exe x -o $NewDir -y $Filename}
            ArgumentList = $NewDir, $FileName
        }
        Invoke-Command @7zip_params
    }
}