Function Find-CommonGroupMembership
{
<#
.NOTES
    Author: Cameron Ratchford
    Version 1.0

.SYNOPSIS 
	Look at group membership for a list of user accounts and find
    any groups which all listed users are a member of

.PARAMETER InputFile
	Full path and file name of the text file containing user accounts to process.
    This file should contain one user account per line.

    Example file contents:
    -------------------------------
    jamie
    blake
    carlos
    

.EXAMPLE
    Find-CommonGroupMembership -InputFile "c:\temp\userlist.txt"
    This command will read all user names in the file 'c:\temp\userlist.txt'
    and check each one for common group membership.  Only common groups directly
    containing the user accounts will be listed.
#>

    [cmdletbinding()]
    param(                        
        [Parameter(Mandatory = $True, HelpMessage="Enter full path to input file name:")]   
            [string]
            $InputFile, 
        [Parameter(Mandatory = $False)]
            [switch]
            $Indirect
        )

write-host "Reading input file " $inputfile
$inputdata = get-content $inputfile

$errorcheck = $false
write-host "Checking user accounts"
foreach ($username in $inputdata) 
{
    $username = $username.trim()
    $user = get-aduser $username
    write-host -ForegroundColor Cyan $InputFile
    write-host -ForegroundColor Magenta $user " is in the list"
    If ($user.count -eq 0) 
    {
        Write-Host "`t Name read from input not found in AD: " $username -ForegroundColor Red
        $errorcheck = $true
    }
    elseif ($user.count -gt 1) 
    {
        Write-Host "`t Name read from input is not unique: " $user.count " items found matching " $username -ForegroundColor Red
        $errorcheck = $true
    }
    else 
    {
        [array]$userlist = $userlist + $username
    }
}

If ($errorcheck -eq $true) 
{
    write-host ""
    write-host "All input file entries detailed in the errors above will be ignored." -ForegroundColor Yellow
    write-host "Only valid names will be processed. Correct any entries in the input file" -ForegroundColor Yellow
    write-host "and run this command again if needed." -ForegroundColor Yellow
    write-host ""
}

$usercount = $userlist.count

If ($usercount -ge 2) 
{
    write-host "Number of user names being processed: " $usercount -ForegroundColor Yellow
    write-host ""

    $comparedata = $userlist | get-aduser | get-adprincipalgroupmembership | group-object name

    foreach ($entry in $comparedata) 
    {
        If ($entry.count -eq $usercount) 
        {
            [array]$output = $output + $entry
        }
    }

    If ($output.count -ge 1) 
    {
        write-host "Groups containing all processed user accounts are:"
        $output | select-object name | sort-object name
    }
    else 
    {
        write-host "No groups were found which contained all processed user accounts"
    }
}

else 
{
    write-host "The input file contains less than two valid entries." -ForegroundColor Yellow
    write-host "No comparison can be performed." -ForegroundColor Yellow
}

$inputdata = $null
$userlist = $null
$output = $null    

}