<#
.SYNOPSIS

    Returns an object with:
        ComputerName (Same as provided in $ComputerName)
        Letter (Volume's drive letter of the disk you're measuring)
        TotalSpace (Total capacity of the disk you're measuring)
        FreeSpace (Capacity remaining of the disk you're measuring)

    Useful to use in scripts where drive capacity should be considered (Transfering files, backups, installing programs, etc.)

.DESCRIPTION
    1. Gets the WMI class Win32_logicalDisk from the computer specified in the $ComputerName parameter
    2. Perofrms math and and string formatting to turn it into a pretty number
    3. Spits out a PSCustom Object you can use in a script (Same way you would a Hash Table)

    Info on hash tables:
    https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_hash_tables?view=powershell-7

    
.PARAMETER ComputerName
    Specifies [string] the hostname of the computer that we are measuring the drive capacity of.

.PARAMETER DriveLetter
    Specifies [string[]] the Windows Drive Letter (Win32_LogicalDisk.DeviceID) you would like the script to return information about. 
    Accepts multiple values.
    Is validated on a set of Alphabet characters.

.PARAMETER All
    Specifies [switch] that you'd like to perform the drive scan on all available disks on the computer specified in $ComputerName .

.EXAMPLE
Getting the drive space infomration on all non-removable drives in a set

PS> Get-FreeDiskSpace -computername dorien -DriveLetter C, D, E | Format-Table

    Output:

    ComputerName Letter TotalSpace FreeSpace ConsumedSpace
    ------------ ------ ---------- --------- -------------
    dorien       C          237.84      77.7        160.14
    dorien       D          465.24    194.99        270.25
    dorien       E          237.91     37.79        200.12

.EXAMPLE
Getting the drive space infomration on all non-removable drives 

PS> Get-FreeDiskSpace -computername jamie -all | Format-Table

    Output:
    
    ComputerName Letter TotalSpace FreeSpace ConsumedSpace
    ------------ ------ ---------- --------- -------------
    jamie        C:          226.7    114.76        111.94
    jamie        D:         465.76     28.68        437.08
    jamie        E:              0         0             0
    jamie        F:        1117.81   1117.36          0.45
.EXAMPLE
Testing to see if there is sufficent drive space on the destination drive to fit the entire contents of the source drive

Code Snippet:

    $SrcDisk = Get-FreeDiskSpace -ComputerName $SourceComputer -DriveLetter E
    $DstDisk = Get-FreeDiskSpace -ComputerName $DestinationComputer -DriveLetter E
    if ($SrcDisk.ConsumedSpace -gt $DstDisk.FreeSpace) 
    {
        Write-Error -Message ("There is not enough space on the destination volume {0}" -f $DstDisk.Letter)
        break
    }
    else
    {
        Copy-Item -Path "\\$SourceComputer\E$" -Destination "\\$DestinationComputer\E$" -Recurse 
    }
.EXAMPLE
Testing to see if there is sufficent drive space on the destination drive to fit the file

Code Snippet:

$File = Get-Item -Path "E:\ISOs\Win10.iso"
$Destination = 
$FileSize = ($file.Length /1GB)



.NOTES
Author:      Cameron Ratchford
Date:        February 25, 2020
Updates:
    February 26, 2020 - Added consumed disk space
#>

Function Get-FreeDiskSpace
{
[CmdletBinding()]
Param(
  [Parameter(Mandatory=$True)]
  [string]$computername,
  [Parameter(Mandatory=$False)]
  [ValidateSet('A','a','B','b','C','c','D','d','E','e','F','f','G','g',
               'H','h','I','i','J','j','K','k','L','l','M','m','N','n',
               'O','o','P','p','Q','q','R','r','S','s','T','t','U','u',
               'V','v','W','w','X','x','Y','y','Z','z')]
  [string[]]$DriveLetter,
  [Parameter(Mandatory=$False)]
  [Switch]$All
  ) 

$output = @()

if ($all)
{
    $DiskObj = Get-WmiObject -Class Win32_logicalDisk -ComputerName $computername | Select-Object -Property DeviceId, DriveType, FreeSpace, size
    foreach ($letter in (Where-Object -InputObject $DiskObj -FilterScript {$_.DriveType -match 3}).DeviceID)
    {
        $DiskObj = Get-WmiObject -Class Win32_logicalDisk -ComputerName $computername | Select-Object -Property DeviceId, DriveType, FreeSpace, size | Where-Object -FilterScript {$_.DeviceID -match $letter}
        $DiskInfo = [PSCustomObject]@{
            ComputerName = $ComputerName
            Letter = $letter
            TotalSpace = [math]::Round(($DiskObj.Size /1GB), 2)
            FreeSpace = [math]::Round(($DiskObj.FreeSpace /1GB), 2)
            ConsumedSpace = [math]::Round((([math]::Round(($DiskObj.Size /1GB), 2)) - ([math]::Round(($DiskObj.FreeSpace /1GB), 2))), 2)
        }

        $output += $DiskInfo
    }
}

else 
{
    foreach ($letter in $DriveLetter)
    {
        $DiskObj = Get-WmiObject -Class Win32_logicalDisk -ComputerName $computername | Select-Object -Property DeviceId, DriveType, FreeSpace, size | Where-Object -FilterScript {$_.DeviceID -match $letter}
        $DiskInfo = [PSCustomObject]@{
            ComputerName = $ComputerName
            Letter = $letter
            TotalSpace = [math]::Round(($DiskObj.Size /1GB), 2)
            FreeSpace = [math]::Round(($DiskObj.FreeSpace /1GB), 2)
            ConsumedSpace = [math]::Round((([math]::Round(($DiskObj.Size /1GB), 2)) - ([math]::Round(($DiskObj.FreeSpace /1GB), 2))), 2)
        }

        $output += $DiskInfo
    } 
}

return $output

}

