
<#
.SYNOPSIS
    Outputs a list of software that is common among all computers listed in the input file
    Extends Get-InstalledSoftware.ps1

.DESCRIPTION
    1. Collects the output of Get-InstalledSoftware into an array for each computer in the input file
    2. Runs comparisson query againsts all arrays for each item in each array, adding array items that match other arrays into a hashtable.
    3. Returns the results of the comparisson, showing which computer each piece of software was installed on, along with other software information.
    
.PARAMETER InputFile
    The string file path of the host file. Accepts .txt and .csv. (csv files must have the computername header)

.PARAMETER All
    A switch parameter that changes the output from "Only full match" to "everything"

.EXAMPLE
    "Get-GroupInstalledSoftware -InputFile C:\temp\visgroup_computers.txt" will display the software that is installed on each of the computers, and on which computers it is installed on

.EXAMPLE
    "Get-GroupInstalledSoftware -InputFile C:\temp\visgroup_computers.csv" will display the software that is installed on each of the computers, and on which computers it is installed on

.EXAMPLE
    "Get-GroupInstalledSoftware -InputFile C:\temp\visgroup_computers.csv | Out-GridView" will display, in gridview, the software that is installed on each of the computers, and on which computers it is installed on    

.NOTES
    Author: Cameron Ratchford
    Date:   January 8, 2020
#>
function Get-GroupInstalledSoftware
{
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)]
        [ValidateScript({Test-Path -Path $_})]
        [string]$InputFile,
        [Parameter(Mandatory=$false)]
        [switch]$All
    )

    $InputFileObj = Get-Item -Path $InputFile
    $ComputerList = @()

    if ($InputFileObj.Extension -eq ".csv")
    {
        $ComputerListFile = Import-CSV -LiteralPath $InputFileObj.FullName 
        foreach ($line in $ComputerListFile.ComputerName)
        {
            $ComputerList += $line
        }
    } 

    if ($InputFileObj.Extension -eq ".txt")
    {
        $ComputerList = Get-Content -Path $InputFileObj.FullName
        foreach ($line in $ComputerList)
        {
            $ComputerList += $line
        }
    } 

    $ResultsHash = @()

    foreach ($Computer in $ComputerList)
    {
        $InstalledSoftware = Get-InstalledSoftware -ComputerName $Computer
        $ResultsHash += @{$Computer = $InstalledSoftware}
    }

    $SoftwareFullList = @()
    foreach ($name in $ResultsHash.Keys)
    {
        $SoftwareList = $ResultsHash.$name
        foreach ($Software in $SoftwareList)
        {
            if ($SoftwareFullList.Name -notcontains $Software.Name)
            {   
                $Software | Add-Member -NotePropertyName $Name -NotePropertyValue $true
                $SoftwareFullList += $Software
            }
            else 
            {
                $i = ($SoftwareFullList.Name).IndexOf($Software.Name)
                if ([bool]($SoftwareFullList[$i].PSobject.Properties.name -notmatch $Name))
                {
                    $SoftwareFullList[$i] | Add-Member -NotePropertyName $Name -NotePropertyValue $true
                }
            }
        }
    }

    $CommonSoftware = @()
    foreach ($Software in $SoftwareFullList)
    {
        $InstalledCount = 0
        foreach ($computer in $ComputerList)
        {
            if ($Software.$Computer -eq $true)
            {
                $InstalledCount++
            }
        }
        ######################################################## TIP ########################################################
        #        To set a different threshold for the default results, modify the the value of $ComputerList.Count          #
        #                 A threshold of 85% would be ($InstalledCount -ge $ComputerList.Count * 0.85)                      #
        #####################################################################################################################
        if ($InstalledCount -eq $ComputerList.Count)
        {
            $Software | Add-Member -NotePropertyName "CommonOnAll" -NotePropertyValue $true
            $CommonSoftware += $Software
        }
    }

    if ($all -eq $true)
    {
        return $SoftwareFullList
    }

    else 
    {
        return $CommonSoftware
    }
    
}

