<#
.SYNOPSIS
    Loads the drivers contained in the provided directory into the provided Wim file.

.DESCRIPTION
    Note: There may be errors reporting that the driver could not be loaded. 
    If you included drivers that don't match the target OS (wrong bitness, wrong version, wrong edition), they will fail to load. This is normal.

    1. Scans the Wim file for available images, providing user with an Out-GridView choice.
    2. Mounts the selected image on the Wim file to the $WorkingDirectory/Mount folder
    2. Executes Add-WindowsDriver with the recurse option to load each driver located in the $FolderPath directory onto the image in Wim file.
    3. Saves the updated image and unmounts the wim file from the $WorkingDirectory folder.
    
.PARAMETER FolderPath
    The folder containing the driver files you wish to load onto the Wim file

.PARAMETER WimPath
    The path of the wim file you wish to load drivers onto

.PARAMETER WorkingDirectory
    The folder where you'd like to mount the wim file so operations can be performed on it (anywhere local with free disk space, a network share will be very slow)

.EXAMPLE
    By itself:

    Install-DriversOnWim -FolderPath 'E:\WDK\Drivers\Dell\T5600' -WimPath 'E:\WDK\Wims\Windows_10\1809\Install.wim' -WorkingDirectory 'E:\WDK\Drivers\Dell\T5600'

.NOTES
    Author:      Cameron Ratchford
    Date:        January 9, 2020  
#>
function Install-DriversOnWim
{
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [string]$FolderPath,

        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [string]$WimPath,

        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [string]$WorkingDirectory
    )

    $MountPath = $WorkingDirectory + "\mount\"

    if (!(test-path -path $MountPath))
    {
        new-item -ItemType Directory -path $MountPath
    }

    $SelectedImage = Get-WindowsImage -ImagePath $WimPath | Select-Object -Property ImageIndex, ImageName | Out-GridView -PassThru -Title "Select Image"

    Mount-WindowsImage -Path $MountPath -ImagePath $WimPath -Index $SelectedImage.ImageIndex

    Add-WindowsDriver -Path $MountPath -Driver $FolderPath -Recurse

    Dismount-WindowsImage -Path $MountPath -Save    
}