<#
.SYNOPSIS
    Initiates a USMT transfer of a specific user from a source computer to a destination computer.
.DESCRIPTION
    1. Copies USMT files to source and destination computer
    2. Creates encryption key based on the secure string that was prompted for after the command was run
    3. Maps a the specified sharepath as a network drive on the source and destination machines
    3. Executes ScanState.exe using WinRM on the source PC, outputting a .mig file to the sharepath
    4. Executes LoadState.exe using WinRM on the destination PC, using the .mig file on the sharepath generated earlier
    5. Cleans up the USMT files and network shares
    
.PARAMETER SourceComputer
    The string hostname of the computer that we are running scanstate.exe on
.PARAMETER DestinationComputer
    The string hostname of the computer that we are running loadstate.exe on
.PARAMETER UserName
    The string username of the user that we're capturing data from
.PARAMETER SharePath
    The string location that scanstate.exe will dump the mig file.
.PARAMETER USMTFilesPath
    The string location of the USMT files required to execute scanstate.exe and loadstate.exe
.PARAMETER Domain
    The string domain name assoicated with the user
.PARAMETER Exclusions
    The boolean switch that will determine whether D:\, E:\, and F:\ will be excluded from scanstate.exe
.PARAMETER SecureKey
    A string input value that scanstate.exe uses to encrypt the mig file

.EXAMPLE
    Invoke-USMT -SourceComputer "cameron" -DestinationComputer "cameronr" -UserName "cameron" -SharePath "\\cameron\E$" -USMTFilesFile "\\ntsvr\apps\Admin\Scripts\USMT\USMTFiles" -Domain "CMGL" -Exlusions $true
    This will initate a backup of cmgl\cameron's files on the computer cameron and restore them on the computer cameronr. The mig file will reside on the \\cameron\E$ share, and it will be using the USMT files located on the \\ntsvr\apps\Admin\Scripts\USMT\USMTFiles share

.EXAMPLE
    Just use the Invoke-CMGUSMT script. It's much easier.

.NOTES
    Version:    1.2
    Author:     Cameron Ratchford
    Created:    October 21, 2019
    Update (Dec 17, 2019): 
                Increased memory allocation for WinRM from 256MB to 4096MB (That should do it, I hope.)
                Logging currently disabled because file locks lead to process terminating exceptions. Will re-add later.
    Update (Jan 21, 2020): 
                Added Repair-WinRM to WinRM tests as part of the Catch statement
                Added todo
    
    TODO: 
        Move Test-Connection to Param Validation 
            Replacing [ValidateNotNullOrEmpty()] with [ValidateScript({Test-Connection -computername $_})]
        Re-implement Write-ToLog
        Remove Unecessary inner-functions
            Invoke-USMT
        Replace Checks for WinRM service with Test-WSMan
        Implement a check for disk space with D and E drives (especially E drive)
        Implement a optional parameter that will read arguments from an encrypted JSON file. 
        Add test for elevated shell, exit with warning if it isn't ran with admin      
#>


function Set-WinRMMemory
{
    [CmdletBinding()]
    Param (
        [String]
        $computername
    )
    $WinRMMemParams = @{
        Computername = $computername
        ScriptBlock = {
            winrm set winrm/config/winrs '@{MaxMemoryPerShellMB="4096"}'
        }
    }
    Invoke-Command @WinRMMemParams -ErrorAction Stop

}

function Invoke-USMT {
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$true)]
        [string]$SourceComputer,
        [Parameter(Mandatory=$true)]
        [string]$DestinationComputer,
        [Parameter(Mandatory=$true)]
        [string]$UserName,
        [Parameter(Mandatory=$true)]
        [string]$SharePath,
        [Parameter(Mandatory=$true)]
        [string]$USMTFilesPath,
        [Parameter(Mandatory=$true)]
        [string]$Domain,
        [Parameter(Mandatory=$true)]
        [bool]$exclusions,
        [Parameter(Mandatory=$true, HelpMessage='Enter encryption key to encrypt the mig file with')]
        [Security.SecureString]$SecureKey,
        [Parameter(Mandatory=$true)]
        [ValidateNotNullOrEmpty()]
        [System.Management.Automation.PSCredential]
        $cred
    )

    #_______________ Tests _______________#

    begin 
    {
        # Test source and destination computers are online
        if (!(Test-Connection -ComputerName $SourceComputer -Count 2))
        {
            Write-Warning -Message "Count not ping $SourceComputer"
            Break
        }
         if (!(Test-Connection -ComputerName $DestinationComputer -Count 2))
        {
            Write-Warning -Message "Count not ping $DestinationComputer"
            Break
        }

        # Test WinRM Service
        $TargetWimRM = get-service -Name WinRM -ComputerName $SourceComputer
        if ($TargetWimRM.Status -eq "Stopped")
        {
            try
            {
                start-service $TargetWimRM
            }
            catch
            {
                try 
                {
                    Repair-WinRM -ComputerName $SourceComputer
                }
                catch 
                {
                    Write-Error "Was not able to start WinRM Service on $SourceComputer"
                    break
                }
            }

        }
        
        $TargetWimRM = get-service -Name WinRM -ComputerName $DestinationComputer
        if ($TargetWimRM.Status -eq "Stopped")
        {
            try
            {
                start-service $TargetWimRM
            }
            catch
            {
                try 
                {
                    Repair-WinRM -ComputerName $DestinationComputer
                }
                catch 
                {
                    Write-Error "Was not able to start WinRM Service on $DestinationComputer"
                    break
                }
            }

        }
    }
    
    process 
    {
        #_______________ Pre-reqs _______________#

        # The script will skip to cleanup on a caught exception if the migfile is not recoverable.
        $SkipToCleanup = $false                
        
        # Copy USMT files to source computer
        if (!($SkipToCleanup))
        {
            Try 
            {
                Copy-Item -Path $USMTFilesPath -Destination "\\$SourceComputer\C$\" -ErrorAction Stop -Recurse -force 
                
            }
            Catch
            {
                Write-Error $_
                $SkipToCleanup = $True 
            }
        }
            
        # Copy USMT files to destination computer
        if (!($SkipToCleanup))
        {
            Try 
            {
                Copy-Item -Path $USMTFilesPath -Destination "\\$DestinationComputer\C$\" -ErrorAction Stop -Recurse -force 
                
            }
            Catch 
            {
                Write-Error $_
                $SkipToCleanup = $true
            }

            $SharePath = $SharePath + "\" + $UserName
        }

        # Create directory path on remote share with $Sharepath
        if (!($SkipToCleanup))
        {
            if (!(test-path -Path $sharepath))
            {
                try 
                {
                    new-item -Path $sharepath -ItemType Directory -ErrorAction Stop > $null
                    
                }
                catch 
                {
                    Write-Error $_

                    Write-host -ForegroundColor "blue" "$Sharepath not created"
                    $SkipToCleanup = $true
                }
            }
        }

        # Create PSSesssion for source and destination computers
        if (!($SkipToCleanup))
        {
            try 
            {    
                $source_sess = New-PSSession -ComputerName $SourceComputer -ErrorAction Stop
            }
            catch
            {
                Write-Error $_
                $SkipToCleanup = $true
            }

            try 
            {
                $dest_sess = New-PSSession -ComputerName $DestinationComputer -ErrorAction Stop
            }
            catch
            {
                Write-Error $_
                $SkipToCleanup = $true
            }
        }

        # Set WinRM memory capacity to 4GB so scanstate and loadstate don't exceed the ps remote shell limits
        if (!($SkipToCleanup))
        {
            try 
            {
                Set-WinRMMemory -computername $SourceComputer
            }
            catch
            {
                write-error $_
                Write-Error $_.InvocationInfo.ScriptLineNumber
            }

            try
            {
                Set-WinRMMemory -computername $DestinationComputer
            }
            catch
            {
                write-error $_
                Write-Error $_.InvocationInfo.ScriptLineNumber
            }
        }
        

        # Convert PSCredential to usable string for net use command
        if (!($SkipToCleanup))
        {
            if ($cred.UserName -match $env:USERDOMAIN)
            {
                $user_combined_path = $cred.UserName
            }
            else 
            {
                $user_combined_path = $env:USERDOMAIN + "\" + $cred.UserName
            }

            $BSTR = [System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($cred.Password)
            $string_pwd = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto($BSTR) 
        }

        # Find an open drive letters on source source
        if (!($SkipToCleanup))
        {
            try
            {
                $source_share_path = Invoke-Command -Session $source_sess -ScriptBlock ${function:Get-OpenNetworkDrive} -ErrorAction Stop
            }
            catch
            {
                Write-Error $_
                $SkipToCleanup = $true
            }
        }
        
     
 
        # Map network drive on source computer
        if (!($SkipToCleanup))
        {
            try
            {
                $source_drivemap_params = @{
                    Session = $source_sess
                    ScriptBlock = { Param ($source_share_path, $sharepath, $user_combined_path, $string_pwd) net use $source_share_path $sharepath /PERSISTENT:YES /USER:$user_combined_path $string_pwd  } 
                    ArgumentList = $source_share_path, $sharepath, $user_combined_path, $string_pwd
                }

                Write-host "Source drivemap variables:"
                Write-host $source_share_path, $sharepath, $user_combined_path, $string_pwd

                Invoke-Command @source_drivemap_params -ErrorAction Stop
                
            }
            catch 
            {
                Write-Error "Problem mapping network drive on $SourceComputer"
                Write-Error $_

                $SkipToCleanup = $true
            }
        }

        # Find an open drive letters on destination source
        if (!($SkipToCleanup))
        {
            try 
            {
                $destination_share_path = Invoke-Command -Session $dest_sess -ScriptBlock ${function:Get-OpenNetworkDrive} -ErrorAction Stop
               
            }
            catch 
            {
                Write-Error $_
                $SkipToCleanup = $true
            }   
        }

        # Map network drive on destination computer
        if (!($SkipToCleanup))
        {
            try 
            {
                $dest_drivemap_params = @{
                    Session = $dest_sess
                    ScriptBlock = { Param ($destination_share_path, $sharepath, $user_combined_path, $string_pwd) net use $destination_share_path $sharepath /PERSISTENT:YES /USER:$user_combined_path $string_pwd } 
                    ArgumentList = $destination_share_path, $sharepath, $user_combined_path, $string_pwd
                }

                Write-host "Destination drivemap variables:"
                Write-host $destination_share_path, $sharepath, $user_combined_path, $string_pwd

                Invoke-Command @dest_drivemap_params -ErrorAction Stop 
                
            }
            catch 
            {
                Write-Error "Problem mapping network drive on $destinationComputer"
                Write-Error $_

                $SkipToCleanup = $true
            }
        }    

        #_______________ Test Drive Capacity _______________#
        if ($exclusions)
        {
            $SrcDisk = Get-FreeDiskSpace -ComputerName $SourceComputer -DriveLetter C
            $DstDisk = Get-FreeDiskSpace -ComputerName $DestinationComputer -DriveLetter C
            if ($SrcDisk.ConsumedSpace -gt $DstDisk.TotalSpace)
            {
                Write-Warning "The destination drive is smaller than the source drive.`nPlease ensure that the user's data will fit on the new disk before proceeding"
                $Prompt = Read-Host "Do you wish to continue? ( [Y] , N )"

                $continue = $true
            
                if ($Prompt -eq "Y" -or $Prompt -eq "y")
                {
                    $continue = $true
                }
            
                elseif ($Prompt -eq "N" -or $Prompt -eq "n") 
                {
                    $continue = $false
                }
            
                if (!($continue))
                {
                    $SkipToCleanup = $true
                }
            }
        }
        else 
        {
            $SrcDiskC = Get-FreeDiskSpace -ComputerName $SourceComputer -DriveLetter C
            $DstDiskC = Get-FreeDiskSpace -ComputerName $DestinationComputer -DriveLetter C
            $SrcDiskD = Get-FreeDiskSpace -ComputerName $SourceComputer -DriveLetter D
            $DstDiskD = Get-FreeDiskSpace -ComputerName $DestinationComputer -DriveLetter D
            $SrcDiskE = Get-FreeDiskSpace -ComputerName $SourceComputer -DriveLetter E
            $DstDiskE = Get-FreeDiskSpace -ComputerName $DestinationComputer -DriveLetter E

            $WarningC = ($SrcDiskC.ConsumedSpace -gt $DstDiskC.FreeSpace)
            $WarningD = ($SrcDiskD.ConsumedSpace -gt $DstDiskD.FreeSpace)
            $WarningE = ($SrcDiskE.ConsumedSpace -gt $DstDiskE.FreeSpace)
            if ($WarningC -or $WarningD -or $WarningE)
            {
                function Select-Color
                {
                    Param ($Warning)
                    if ($Warning)
                    {
                        return "Red"
                    }
                    else
                    {
                        return "Green"
                    }
                }
                Write-Host "`n`n"
                Write-Warning "One or more of the destination drives do not have enough capacity to store the contents of the source drive.`nPlease ensure that the user's data will fit on the new disk before proceeding`n"
                Write-Host -ForegroundColor (Select-Color -Warning $WarningC) "C:\ Warning = $WarningC"
                Write-Host -ForegroundColor (Select-Color -Warning $WarningD) "D:\ Warning = $WarningD"
                Write-Host -ForegroundColor (Select-Color -Warning $WarningE) "E:\ Warning = $WarningE"
                Write-Host "`n`nA warning for the C drive is not often an issue.`nIf you received a warning for the D or E drive, there is a very high likelihood that the script will fail."
                $Prompt = Read-Host "`nDo you wish to continue? ( [Y] , N )"

                $continue = $true
            
                if ($Prompt -eq "Y" -or $Prompt -eq "y")
                {
                    $continue = $true
                }
            
                elseif ($Prompt -eq "N" -or $Prompt -eq "n") 
                {
                    $continue = $false
                }
            
                if (!($continue))
                {
                    $SkipToCleanup = $true
                }
            }
        }

        #_________Check For Windows 7________#
        $CheckForWin7 =  Get-WmiObject -ComputerName $SourceComputer -class Win32_OperatingSystem | Select-Object Caption | Where-Object -FilterScript {$_.Caption -match "Microsoft Windows 7"}

        if ([bool]$CheckForWin7)
        {
            $TargetWindows7 = $True
            Write-Host "Windows 7 Detected"
        }

        else 
        {
            $TargetWindows7 = $False
            Write-Host "Windows 10 Detected"
        }

        # If Win7, generate a config.xml file
        if($TargetWindows7)
        {
            $win7config_params = @{
                Session = $source_sess
                ScriptBlock = { 
                    Param ($source_share_path, $user_combined_path) 
                    c:\USMTFiles\scanstate.exe /targetwindows7 /genconfig:$source_share_path\config.xml /i:c:\usmtfiles\migdocs.xml /i:c:\usmtfiles\migapp.xml /i:c:\usmtfiles\excludefiles.xml /v:13
                }
                ArgumentList = $source_share_path, $user_combined_path
            }

            Invoke-Command @win7config_params
        }

        #_______________ Decide Scanstate Params _______________#
        # Exclusions and Win7
        if($exclusions -and $TargetWindows7)
        {
            $scan_params = @{
                Session = $source_sess
                ScriptBlock = { 
                    Param ($source_share_path, $user_combined_path, $SecureKey) 
                    c:\USMTFiles\scanstate.exe $source_share_path /o /vsc /config:$source_share_path\config.xml /i:c:\usmtfiles\migdocs.xml /i:c:\usmtfiles\migapp.xml /i:c:\usmtfiles\excludefiles.xml /v:13 /ui:$user_combined_path /c /efs:copyraw /localonly /encrypt /key:$SecureKey /listfiles:c:\usmtfiles\listfiles.txt /ue:*\*
                }
                ArgumentList = $source_share_path, $user_combined_path, $SecureKey
            }
        }

        # No Exclusions and Win7
        elseif(!($exclusions) -and $TargetWindows7)
        {
            $scan_params = @{
                Session = $source_sess
                ScriptBlock = { 
                    Param ($source_share_path, $user_combined_path, $SecureKey) 
                    c:\USMTFiles\scanstate.exe $source_share_path /o /vsc /config:$source_share_path\config.xml /i:c:\usmtfiles\migdocs.xml /i:c:\usmtfiles\migapp.xml /v:13 /ui:$user_combined_path /c /efs:copyraw /localonly /encrypt /key:$SecureKey /listfiles:c:\usmtfiles\listfiles.txt /ue:*\*
                }
                ArgumentList = $source_share_path, $user_combined_path, $SecureKey
            }
        }

        # No Exclusions and Win10
        elseif(!($exclusions) -and !($TargetWindows7))
        {
            $scan_params = @{
                Session = $source_sess
                ScriptBlock = { 
                    Param ($source_share_path, $user_combined_path, $SecureKey) 
                    c:\USMTFiles\scanstate.exe $source_share_path /o /vsc /i:c:\usmtfiles\migdocs.xml /i:c:\usmtfiles\migapp.xml /v:13 /ui:$user_combined_path /c /efs:copyraw /localonly /encrypt /key:$SecureKey /listfiles:c:\usmtfiles\listfiles.txt /ue:*\*
                }
                ArgumentList = $source_share_path, $user_combined_path, $SecureKey
            }
        }

        # Exclusions and Win10
        elseif($exclusions -and !($TargetWindows7))
        {
            $scan_params = @{
                Session = $source_sess
                ScriptBlock = { 
                    Param ($source_share_path, $user_combined_path, $SecureKey) 
                    c:\USMTFiles\scanstate.exe $source_share_path /o /vsc /i:c:\usmtfiles\migdocs.xml /i:c:\usmtfiles\migapp.xml /i:c:\usmtfiles\excludefiles.xml /v:13 /ui:$user_combined_path /c /efs:copyraw /localonly /encrypt /key:$SecureKey /listfiles:c:\usmtfiles\listfiles.txt /ue:*\*
                }
                ArgumentList = $source_share_path, $user_combined_path, $SecureKey
            }
        }

        #_______________ Execute Scanstate via WinRM _______________#
        if (!($SkipToCleanup))
        {
            try 
            {
                Invoke-Command @scan_params -ErrorAction Stop
            }
            catch 
            {
                Write-Error "USMT Scanstate failed"
                Write-Error $_
                $SkipToCleanup = $true
            }
        }

        #_______________ Decide Loadstate Params _______________#
        if ($TargetWindows7)
        {
            $load_params = @{
                Session = $dest_sess
                ScriptBlock = {
                    Param ($destination_share_path, $user_combined_path, $SecureKey) 
                    c:\USMTFiles\loadstate.exe $destination_share_path /config:$destination_share_path\config.xml /i:c:\usmtfiles\migdocs.xml /i:c:\usmtfiles\migapp.xml /v:13 /ui:$user_combined_path /c /decrypt /key:$SecureKey
                }
                ArgumentList = $destination_share_path, $user_combined_path, $SecureKey
            }
        }

        else 
        {
            $load_params = @{
                Session = $dest_sess
                ScriptBlock = {
                    Param ($destination_share_path, $user_combined_path, $SecureKey) 
                    c:\USMTFiles\loadstate.exe $destination_share_path /i:c:\usmtfiles\migdocs.xml /i:c:\usmtfiles\migapp.xml /v:13 /ui:$user_combined_path /c /decrypt /key:$SecureKey
                }
                ArgumentList = $destination_share_path, $user_combined_path, $SecureKey
            }
        }

        #_______________ Execute Loadstate via WinRM _______________#
        if (!($SkipToCleanup))
        {
            try 
            {
                Invoke-Command @load_params -ErrorAction Stop
            }
            catch 
            {
                Write-Error $_
                Write-Error "USMT Loadstate failed"
                # Has to break or else cleanup will run
                break
            }
        }
        
        #_______________ Clean up _______________#
        try
        {
            # Remove USMT files from Source Computer
            if (test-path -Path "\\$SourceComputer\C$\USMTFiles")
            {
                Remove-Item \\$SourceComputer\C$\USMTFiles -Force -Recurse -ErrorAction Stop
            }

            # Remove USMT files from Destination Computer
            if (test-path -Path "\\$DestinationComputer\C$\USMTFiles")
            {
                Remove-Item \\$DestinationComputer\C$\USMTFiles -Force -Recurse -ErrorAction Stop
            }

            # Remove clean up Mig file
            if (test-path -Path $sharepath)
            {
                Remove-Item $sharepath -Force -Recurse -ErrorAction Stop
            }

            # Unmap USMT Share from Source Computer
            $source_drive_unmap= @{
                Session = $dest_sess
                ScriptBlock = { Param ($source_share_path) net use $source_share_path /DELETE} 
                ArgumentList = $source_share_path
            }

            Invoke-Command @source_drive_unmap -ErrorAction Stop 

            # Unmap USMT Share from Destination Computer
            $destination_drive_unmap= @{
                Session = $dest_sess
                ScriptBlock = { Param ($destination_share_path) net use $destination_share_path /DELETE } 
                ArgumentList = $destination_share_path
            }

            Invoke-Command @destination_drive_unmap -ErrorAction Stop 
                
        }
        catch
        {
            Write-Error $
            Write-Error "Script did not clean up Gracefully"
        }

     }
}

