<#
.SYNOPSIS
    Automates the process of provisioning a user for remote access through CMGL's citrix gateway.

.DESCRIPTION
    1. Tests if user is in AD, part of Citrix-CMG-CalgaryRemoteAccess or Citrix-CMG-RemoteDesktopRD 
    2. Checks to see if the user is already in the enabled_users.txt file, adding them if they are not
    3. Copies PSExec.exe to the local machine
    4. Executes the sync_users.cmd batch script remotely on 2factorsvr
    5. Generates the QR code, copying it to the local machine
    6. Generates the OTP Prefix
    7. Formats the email message, attaching the QR code, and providing instructions on how to process, providing the OTP prefix and linking necessary documentation from the how-to sharepoint site.
    
.PARAMETER UserName
    The username of the user that you'd like to set up on citrix


.NOTES
    Author:                 Cameron Ratchford
    Date:                   March 12, 2020  
#>


function New-CMGCitrixUser
{
    [CmdletBinding()]
    param (
        [Parameter(Mandatory=$true)]
        [string]$UserName
    )

    # Variables that might need to be changed
    $PSExecSource = "\\ntsvr\apps\Admin\Scripts\USMT\USMTFiles\PsExec64.exe"

    ## Tests

    # Test user in AD
    try
    {
        $User = Get-ADUser -Identity $UserName -Properties * > $null
        Write-host -ForegroundColor Green "User is in AD"
    }
    catch
    {
        Write-warning "User is not in AD`nExiting..."
        break
    }

    # Test if user is part of the Citrix Group
    try 
    {
        $groupmembership = Get-ADGroupMember -Identity Citrix-CMG-CalgaryRemoteAccess | Select-Object samaccountname
        $groupmembership += Get-ADGroupMember -Identity Citrix-CMG-RemoteDesktopRD | Select-Object samaccountname
        if ($groupmembership.samaccountname -contains $username)
        {
            Write-host -ForegroundColor Green "User is part of a Citrix group"
        }
        else 
        {
            Write-warning "User is not part of a Citrix group`nExiting..."
            break
        }
    }

    catch
    {
        Write-warning "Problems collecting user group membership`nExiting..."
        break
    }

    $me = get-aduser -Identity $ENV:USERNAME -Properties *

    # Test if user is already in enabled_users.txt
    try 
    {
        $EnabledUsers =  Get-Content -Path "\\2factorsvr\c$\OTP_Usermgmt\enabled_users.txt" 
        if ($EnabledUsers -contains $UserName)
        {
            Write-Warning "$UserName already in enabled_users.txt`nExiting..."
            break
        }
        else 
        {
            $EnabledUsers = $EnabledUsers | Where-Object -FilterScript {$_ -ne "****END****"}
            $EnabledUsers += $UserName
            $EnabledUsers += "****END****"
            $EnabledUsers | Out-File -FilePath "\\2factorsvr\c$\OTP_Usermgmt\enabled_users.txt" -Force
        }
    
    }
    catch
    {
        Write-Error "Could not get list of enabled users"
        break
    }

    # Copy PSExec to the computer running this script

    try
    {
        Copy-Item -Path $PSExecSource -Destination $PSExecHost -Force
        copy-item -Path "\\ntsvr\apps\Admin\Scripts\USMT\USMTFiles\PsExec64.exe" -Destination "C:\Windows\System32\" -Force
    }
    catch
    {
        Write-Error $_.message
        Write-Warning -Message "Ensure that you have access to $PSExecSource and the PSExec64.exe is located there`n"
    }

    # Execute sync_user.cmd via PSExec
    try
    {
        $PSExecParams = @{
            ScriptBlock = { Param($user, $string_pwd) C:\Windows\System32\PsExec64.exe \\2factorsvr -d -nobanner c:\OTP_Usermgmt\sync_user.cmd}
            ArgumentList = $User, $string_pwd
        }    
    
        Invoke-Command @PSExecParams -ErrorAction Stop
    }
    catch
    {
        Write-Error -Message "`nFailed to run sync_user script remotely"
        $exit = $true
    }
    # Cleanup
    finally 
    {
        Remove-Item -Path "c:\Windows\System32\PsExec64.exe"
    }

    if ($exit)
    {
        break
    }
    else 
    {
        $LocalMachine = $ENV:COMPUTERNAME
        $QRCodeParams = @{
            ScriptBlock = { Param($UserName, $LocalMachine) C:\Windows\System32\PsExec64.exe \\2factorsvr -nobanner c:\multiotp\windows\multiotp.exe -qrcode $UserName \\$LocalMachine\c$\Scripts\$UserName`.png}
            ArgumentList = $UserName, $LocalMachine
        }    
        Invoke-Command @QRCodeParams -ErrorAction Stop

        $Prefix = -Join ($User.SID.Value)[-3..-1]

        Send-MailMessage -SmtpServer "mail.cmgl.ca" -Attachments "\\$LocalMachine\c$\Scripts\$UserName`.png" -To $User.EmailAddress -From $me.EmailAddress -Subject "Citrix Setup instructions" -Body "
To authenticate yourself on CMG network, you will have to use a strong authentication password also known as an OTP code. This code will be generated by your smartphone and will change every time you use it.

The generated code is valid for the following account: $username
and you will have to use the following prefix PIN: $prefix

In order to install the code generator on your smartphone, you will have to follow the next three steps. If you don't have smart phone, please install winauth(https://winauth.com/download/) on windows pc and use secret seed for google below.

1) Install Google Authenticator on your mobile device.
If your smartphone does not have Google Authenticator app yet, please install it from Google Play Store for Android device or Apple App Store for Iphone device.

2) Link the token created to your username.
The last step consists in inserting your personal token key generator into the application. Simply scan the attached QRcode ($username.png).

3)  Congratulations, you are now ready to authenticate yourself on CMG network. If you are using this to access CMG's Citrix portal, please read the 'Setting up Citrix Receiver for Remote Access' documentation linked below.
'https://howto.cmgl.ca/Shared%20Documents/Remote%20Access%20Guides/Setting%20up%20Citrix%20Receiver%20for%20Remote%20Access.pdf'
        " 
    }
}

