
<#
.SYNOPSIS
    Presents user with a winforms dialog for either file selection or folder selection. Returns the selected item path.

    Can be Used in place of read-host when looking for file or folder paths

.DESCRIPTION
    Has the ability to switch between file/folder selection
    Able to filter file types and specify initial directory

    1. Creates a winform object with the specified parameters
    2. Runs the OpenFileDialog method from the object
    3. Returns the file or folder path that the user chose in the dialog

.PARAMETER InitialDirectory
    Not usable with the Directory option
    Specifies which directory you'd like the file dialog to open in. Defaults to C:\

.PARAMETER FileTypeFilter
    Not usable with the Directory option
    Specifies the filter to be applied to the file dialog. Defaults to "All files (*.*)| *.*".
    The section before the pipe (|) specifies what the filter selection box will display, the section after the pipe is the filter that is actually applied.

.PARAMETER Directory
    Changes the file dialog to a folder selection dialog.

.EXAMPLE
    Getting the path of a file of any type:

    $File = Open-FileDialog
    Get-Content -Path $File

.EXAMPLE
    Getting the path of a csv file where the path is likely to be in the D:\ drive:

    $File = Open-FileDialog -InitialDirectory "D:\" -FileTypeFilter "CSV files (*.csv)| *.csv"
    Get-Service | Export-CSV -Path $File -Append 

.EXAMPLE
    Getting the path of a folder:

    $Folder = Open-FileDialog -Directory 
    $FolderContent = Get-ChildItem -Path $Folder -Recurse

.NOTES
    Author:      Cameron Ratchford
    Date:        March 5, 2020
#>

Function Open-FileDialog
{  
    [CmdletBinding()]
    param(
        [Parameter(Mandatory=$false)]
        [string]$InitialDirectory="C:\",
        [Parameter(Mandatory=$false)]
        [string]$FileTypeFilter="All files (*.*)| *.*",
        [Parameter(Mandatory=$False)]
        [Switch]$Directory
    )

    [System.Reflection.Assembly]::LoadWithPartialName("System.windows.forms") | Out-Null

    if ($Directory)
    {
        $OpenFolderDialog = New-Object System.Windows.Forms.FolderBrowserDialog
        $OpenFolderDialog.ShowNewFolderButton = $True
        $OpenFolderDialog.Description = "Select a folder"
        $OpenFolderDialog.rootfolder = "MyComputer"
    
        if($OpenFolderDialog.ShowDialog() -eq "OK")
        {
            $folder += $OpenFolderDialog.SelectedPath
        }
        return $folder
    }
    else
    {
        $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
        $OpenFileDialog.Title = "Select a file"
        $OpenFileDialog.InitialDirectory = $InitialDirectory
        $OpenFileDialog.filter = $FileTypeFilter

        if($OpenFileDialog.ShowDialog() -eq "OK")
        {
            return $OpenFileDialog.FileName
        }

    }
} 
