<#
.SYNOPSIS
    Returns the InnerException of the provided error message

.DESCRIPTION
    Taken from https://leanpub.com/thebigbookofpowershellerrorhandling/read Figure 4.3
    By Dave Wyatt

    VERY USEFUL for Try / Catch blocks, pinpointing the exact type of Exception thrown. 
    Paste the output of this function into Catch[EXCEPTION.TYPE.HERE]

    1. Drills down into the properties of the System.Exception object, finding the InnerException property.
    2. Formats the string and writes it to stdout.
    
.PARAMETER Exception
    The System.Exception object you wish to know the InnerException property of. 
    Can be $error[i], or the pipeline variable $_ in a try/catch block. (Since the Try statement automatically pipes the exception to the Catch statement)

.EXAMPLE
    After the fact, inline on a PowerShell console:

    PS> Function-ThatDoesntExist
    """
    Function-ThatDoesntExist : The term 'Function-ThatDoesntExist' is not recognized as the name of a cmdlet, function,
    script file, or operable program. Check the spelling of the name, or if a path was included, verify that the path is    correct and try again.
    At line:1 char:1
    + Function-ThatDoesntExist
    + ~~~~~~~~~~~~~~~~~~~~~~~~
    + CategoryInfo          : ObjectNotFound: (Function-ThatDoesntExist:String) [], CommandNotFoundException
    + FullyQualifiedErrorId : CommandNotFoundException
    """

    PS> Show-ExceptionType -Exception $Error[0]
    """
    System.Exception
    """

.EXAMPLE
    In a Try / Catch block:

    Try {
        Invoke-Command -ComputerName $ServerName -ScriptBlock {Powershell -File "C:\Scripts\TestScript.ps1"}
    }

    # Catch a known exception type
    Catch[System.Management.Automation.Remoting] {
        Repair-WinRM -ComputerName $ServerName
    }

    # Return the uncaught exception type
    Catch {
        Show-ExceptionType -Exception $_
    }

.NOTES
    Not my script. 
    Author:                 Dave Wyatt
    Date:                   September 28, 2018  
#>
function Show-ExceptionType
{
        [CmdletBinding()]
        param (
            [Parameter(Mandatory)]
            [System.Exception]
            $Exception
        )

        $Indent = 0
        $E = $Exception

        While($E)
        {
            Write-Host ("{0,$Indent}{1}" -f '', $E.GetType().FullName)
            $Indent += 2
            $E = $E.InnerException
        }
} Export-ModuleMember -Function Show-ExceptionType
