﻿<#
.SYNOPSIS
    Gather monitor serial numbers and models from hosts
.DESCRIPTION
    Extends "\\ntsvr\apps\Admin\Scripts\serial.bat" and "\\ntsvr\apps\Admin\Scripts\monitor_sn.vbs"
    Adds functonality for querying a large amount of hosts either by OU or using a hostnames.txt file
.PARAMETER HostnamesFilePath
    The path to the file containing a list of hostnames to query
.PARAMETER OU
    Specifies the OU to query for a list of device names to run this script on
.PARAMETER OutputPath
    Specifies the output csv file location and name. Default is .\results.csv
.EXAMPLE
    C:\PS> .\get-monitors.ps1 -HostnamesFilePath c:\users\jimmy\desktop\hosts.txt -OutputPath C:\users\jimmy\desktop\results.csv
    This will gather all info on connected monitors from hosts listed in hosts.txt then output it to a file called results.csv
.EXAMPLE
    c:\PS> .\get-monitors.ps1 -OU "OU=Desktops,DC=cgy,DC=cmgl,DC=ca"
    This will gather info from all workstations in the Desktops OU in cgy.cmgl.ca domain
.NOTES
    Author: Dorien McClure
    Date:   November 22, 2018    
#>

######################################################################################################
#
#  Dorien McClure
#  Get-Monitors.PS1
#  Extends "\\ntsvr\apps\Admin\Scripts\serial.bat" and "\\ntsvr\apps\Admin\Scripts\monitor_sn.vbs"
#  Adds functonality for querying a large amount of hosts either by OU or using a hostnames.txt file
#  Nov 22nd 2018
#
######################################################################################################

function get-monitors
    {

    #Defining command line params
    param (
        [string]$HostnamesFilePath,
        [string]$OU,
        [string]$OutputPath = ".\results.csv"
    )

    #Check to see if one of the two mandatory parameters has been defined
    if (!$HostnamesFilePath -and !$OU) {
        throw "You have not specified either a hostnames file or an OU to query please do so using -HostnamesFilePath or -OU"
        exit(1)
    }

    #Set the $hostnames source to the selected parameter
    if ($HostnamesFilePath) {
        $hostnames = Get-Content $HostnamesFilePath
    }
    elseif ($OU) {
        $hostnames = (Get-ADComputer -filter * -SearchBase $OU | select name).name
    }

    #Initialize the inventory array
    $inv_array = @()

    #Loop through each name in $hostnames, record the hostname, serial, and monitor models and serials then add them to the inventory array
    foreach ($name in $hostnames) {

        Write-Host ""
        write-host "Scanning computer: $name"

        #Call serial.bat written by Hong to gather the info, then split it into an array of strings
        $raw_output = \\ntsvr\apps\Admin\Scripts\serial.bat $name.Trim()
        $split_output = $raw_output.Split(':')
    
        #Initialie the result array to add to the table
        $result = "" | Select Hostname,Serial,M1Model,M1Serial,M2Model,M2Serial,M3Model,M3Serial,M4Model,M4Serial,M5Model,M5Serial,M6Model,M6Serial

        #Detect an unreachable host and record that in the array
        if($split_output[1] -eq "Abstract") {
            $result.Hostname = $name
            $result.Serial = "Device Unreachable"
            $result.M1Model = ""
            $result.M1Serial = ""
            $result.M2Model = ""
            $result.M2Serial = ""
            $result.M3Model = ""
            $result.M3Serial = ""
            $result.M4Model = ""
            $result.M4Serial = ""
            $result.M5Model = ""
            $result.M5Serial = ""
            $result.M6Model = ""
            $result.M6Serial = ""
        }
        #Otherwise assign the output to the values in the array
        else {
            $result.Hostname = $name
            $result.Serial = $split_output[1]
            $result.M1Model = $split_output[2]
            $result.M1Serial = $split_output[3]
            $result.M2Model = $split_output[4]
            $result.M2Serial = $split_output[5]
            $result.M3Model = $split_output[6]
            $result.M3Serial = $split_output[7]
            $result.M4Model = $split_output[8]
            $result.M4Serial = $split_output[9]
            $result.M5Model = $split_output[8]
            $result.M5Serial = $split_output[9]
            $result.M6Model = $split_output[8]
            $result.M6Serial = $split_output[9]
        }

        #Add the results array to the inventory array
        $inv_array += $result

    }

    #Show the array as a table and output it to the desired csv file
    $inv_array | ft
    $inv_array | Export-Csv $OutputPath

}